#!/usr/bin/python -tt 
# encoding: utf-8
# By Holger Rapp <SirVer@gmx.de>
#
# January 2009
#
# Released into Public domain
#
import wx
import sys
import tempfile
import os
import collections
import appscript as AS

import re

class OmniFocusInterface(object):
    def __init__( self ):
        self._of = AS.app("OmniFocus")

    def get_all_tasks( self ):
        """
        Returns some properties from all tasks
        
        """
        d = self._of.documents[0]
        rv = collections.defaultdict(list) 

        def add_tasks(t,rv,pn = None):
            rv["task"].extend( t() )
            rv["name"].extend( t.name() )
            rv["completed"].extend( t.completed() )
            rv["context"].extend( t.context.name() )
            if pn is not None:
                rv["project"].extend( t.count() * [pn,] )
            else:
                rv["project"].extend( t.containing_project.name() )

        
        add_tasks(d.inbox_tasks,rv)

        def _walk_folder( f,rv ):
            for proj in f.projects():
                pn = proj.name()
                add_tasks(proj.tasks,rv,pn)

            for nf in f.folder():
                _walk_folder(nf,rv)

        for f in d.folders():
            _walk_folder(f,rv)

        return rv
    
    def find_project( self, name ):
        """
        Finds the given project in the folder hierachy
        """
        d = self._of.documents[0]

        def _walk_folder( f ):
            n = f.projects.name()

            if name in n:
                return f.projects[name]
            for nf in f.folder():
                rv = _walk_folder(nf)
                if rv is not None:
                    return rv

            return None

        for f in d.folders():
            rv = _walk_folder(f)
            if rv is not None:
                return rv
    
    def add_task( self, p, taskname, note ):
        """
        Add a task to the given project
        """
        p.make( new=AS.k.task, with_properties = {
            AS.k.name: taskname,
            AS.k.note: note }
        )
    
DEFAULT_SPACE = 20
class MarkDoneWindow( wx.Frame ):
    def __init__( self, parent, id, size ):
        """
        This program lists all task in OF in a window
        and marks them done when selected. This kills searching
        for it in OF and will hopefully boost productivity
        """
        wx.Frame.__init__( self, parent, id =id,
            title ="Mark Action done", name = "MarkActionDone",
            size = size )
        self._of = OmniFocusInterface()

        menuBar = wx.MenuBar()
        
        # Add Menu (Mainly for closing of Window)
        menu1 = wx.Menu()
        menu1.Append(104, "&Close\tCtrl+W", "Close Window")
        menu1.Append(105, "&Close\tCtrl+E", "Edit Task")
        menuBar.Append(menu1, "&General")
        self.Bind(wx.EVT_MENU, self.CloseWindow, id=104)
        self.Bind(wx.EVT_MENU, self.EditTask, id=105)
        self.SetMenuBar( menuBar )

        panel = wx.Panel( self )
        vsizer = wx.BoxSizer( wx.VERTICAL )
        # TextEntry
        hsizer = wx.BoxSizer( wx.HORIZONTAL )
        self.textEntry = wx.TextCtrl( panel )
        hsizer.Add( self.textEntry, 1, wx.EXPAND )
        self.Bind( wx.EVT_TEXT, self.OnTextChanged, self.textEntry )
        self.Bind( wx.EVT_TEXT_ENTER, self.OnDoneClicked, self.textEntry )
        # add it 
        vsizer.Add( (0,DEFAULT_SPACE), 0 )
        vsizer.Add( hsizer, 0, wx.EXPAND )

        # List 
        l = self.listCtrl = wx.ListCtrl( panel, style = wx.LC_SORT_ASCENDING | wx.LC_REPORT | wx.LC_SINGLE_SEL)
        vsizer.Add( (0,DEFAULT_SPACE), 0 )
        vsizer.Add( self.listCtrl, 1, wx.EXPAND )
        self.Bind( wx.EVT_LIST_ITEM_ACTIVATED, self.OnDoneClicked, l)
        l.InsertColumn(0,"Name")
        l.InsertColumn(1,"Context")

        # Update the list
        self._regexpr = re.compile(".*",re.IGNORECASE)
        self._tasks = self._of.get_all_tasks()
        self._fill()
        
        l.SetColumnWidth(0, 800)
        l.SetColumnWidth(1, wx.LIST_AUTOSIZE)

        # Informations
        text = wx.StaticText( panel, label = "Nr of tasks: %i" % l.GetItemCount() )
        vsizer.Add( (0,DEFAULT_SPACE), 0 )
        vsizer.Add( text, 0, wx.LEFT )
        vsizer.Add( (0,DEFAULT_SPACE), 0 )

        # OK Button
        btn = wx.Button( panel, label = "Done" )
        self.Bind( wx.EVT_BUTTON, self.OnDoneClicked, btn )
        btn.SetDefault()
        vsizer.Add( (0,DEFAULT_SPACE), 0 )
        vsizer.Add( btn, 0, wx.CENTER )
        vsizer.Add( (0,DEFAULT_SPACE), 0 )

        # Two spaces, and the hsizer
        hsizer = wx.BoxSizer( wx.HORIZONTAL )
        hsizer.Add( (DEFAULT_SPACE,0), 0 )
        hsizer.Add( vsizer, 1, wx.EXPAND )
        hsizer.Add( (DEFAULT_SPACE,0), 0 )
       
       
        panel.SetSizer( hsizer )
        panel.SetAutoLayout( True )
        panel.Fit()

        # self.SetSizeHintsSz( self.GetSize())
        self.Show(True)
       
    def _fill( self ):
        l = self.listCtrl
        l.DeleteAllItems()
        t = self._tasks
        for name,project,context,done,task in zip(t["name"],t["project"],t["context"],t["completed"],t["task"]):
            if done:
                continue
            if not self._regexpr.search(unicode(project) + unicode(name)):
                continue

            l.Append((unicode(project) + u' • ' + unicode(name),unicode(context)) )
        
        l.Select(0,True)

    def OnTextChanged( self, event ):
        text = self.textEntry.GetValue()
        if len(text):
            regstr = r'.*'.join( c for c in text )
        else:
            regstr = r'.*'
        self._regexpr = re.compile(regstr,re.IGNORECASE)
        self._fill()

    def OnDoneClicked( self, event ):
        task = self._get_currently_selected_task()
        task.completed.set(True)
        wx.Exit()
        return 
    
    def EditTask( self, event ):
        """
        Edit the current task (only the note so far) in MacVim
        """
        task = self._get_currently_selected_task()
        note = task.note()

        # Make a temporary file
        id,fp = tempfile.mkstemp()
        f = open(fp,"wb")
        f.write(note.encode("utf-8"))
        f.close()
        os.environ["PATH"] += ":~/bin:/usr/local/bin"
        
        # Launch vim on it
        # HACK: extend the path to that mvim is found
        os.system("mvim -f %s" % fp)

        # Reread the file
        new_note = open(fp).read().decode("utf-8")

        # Set the task comment, if necessary
        if note != new_note:
            task.note.set(new_note)
            wx.Exit()

        # delete the tmpfile
        os.unlink(fp)
       

    def CloseWindow( self, event ):
        self.Close()
    
    def _get_currently_selected_task(self):
        idx = self.listCtrl.GetFirstSelected()
        if idx == -1:
            return

        text = self.listCtrl.GetItem(idx).GetText()
        tidx = [ (unicode(project) + u' • ' + unicode(name)) for project, name in zip(
            self._tasks["project"],self._tasks["name"]) ].index(text)
        task = self._tasks["task"][tidx]
        return task

def main( argv ):
    "Show the window"

    app = wx.PySimpleApp()
    frame = MarkDoneWindow(None,wx.ID_ANY, 
        size = (1020,600),  # Good for my 1280x800 resolution
        )
        
    frame.Centre()
    app.MainLoop()
    

if __name__ == "__main__":
    import sys
    
    main(sys.argv)

